@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: objectbase.cc,v 1.7 2006/04/12 08:12:11 murrayc Exp $ */
+/* $Id: objectbase.cc,v 1.8 2006/11/10 02:24:49 murrayc Exp $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -22,6 +22,7 @@
 
 #include <glibmm/quark.h>
 #include <glibmm/objectbase.h>
+#include <glibmm/propertyproxy_base.h> //For PropertyProxyConnectionNode
 
 
 namespace
@@ -255,6 +256,23 @@ void ObjectBase::get_property_value(const Glib::ustring& property_name, Glib::Va
   g_object_get_property(const_cast<GObject*>(gobj()), property_name.c_str(), value.gobj());
 }
 
+void ObjectBase::connect_property_changed(const Glib::ustring& property_name, const sigc::slot<void>& slot)
+{
+  // Create a proxy to hold our connection info
+  // This will be deleted by destroy_notify_handler.
+  PropertyProxyConnectionNode* pConnectionNode = new PropertyProxyConnectionNode(slot, gobj());
+
+  // connect it to gtk+
+  // pConnectionNode will be passed as the data argument to the callback.
+  // The callback will then call the virtual Object::property_change_notify() method,
+  // which will contain a switch/case statement which will examine the property name.
+  const Glib::ustring notify_signal_name = "notify::" + property_name;
+  pConnectionNode->connection_id_ = g_signal_connect_data(gobj(),
+         notify_signal_name.c_str(), (GCallback)(&PropertyProxyConnectionNode::callback), pConnectionNode, 
+         &PropertyProxyConnectionNode::destroy_notify_handler,
+         G_CONNECT_AFTER);
+}
+
 
 bool _gobject_cppinstance_already_deleted(GObject* gobject)
 {
